# coding: latin-1
"""
Diese Datei ist Teil der Stifte-und-Muse-Portierung fr Python,
entstanden im Rahmen der Diplomarbeit von Ingo Linkweiler,
verfentlicht unter der Creative Commons Lizenz by-nc-sa,
naehere Informationen dazu unter
http://creativecommons.org/licenses/by-nc-sa/3.0/de/.

 Erste Version: 14.01.2002 - 11.03.2003
 Autor: i.linkweiler@gmx.de
"""

from stift import *
import math

# Konstanten
weiss=(255,255,255)
schwarz=(0,0,0)
rot=(255,0,0)
gruen=(0,255,0)
blau=(0,0,255)
gelb=(255,255,0)
grau=(100,100,100)

WEISS=(255,255,255)
SCHWARZ=(0,0,0)
ROT=(255,0,0)
GRUEN=(0,255,0)
BLAU=(0,0,255)
GELB=(255,255,0)
GRAU=(100,100,100)

FETT=1
KURSIV=2
NORMAL=0
STANDARDSTIL=0
STANDARDSCHRIFT="default.ttf"

DURCHSICHTIG=0
GEFUELLT=1

class Buntstift(Stift):
    """
    Der Buntstift ist ein Zeichenwerkzeug, dass sich auf dem Bildschirm bewegen kann.
    Er hat eine Position (Ursprung ist links oben) und eine Richtung (0'=rechts, Drehsinn math. positiv.)
    Der Stift kennt zwei Zustnde: gesenkt (dann wird geschrieben) und gehoben (dann wird
    der Stift bewegt, ohne zu schreiben).
    Zum Zeichnen kennt er zwei Modi: 'normal' zum Schreiben, und 'radieren' zum Loeschen bzw. Ausradieren.
    Er hat eine Farbe (Konstanten: schwarz, weiss, rot, gruen, blau, gelb);
    sowie einen Schriftstil.
    """

    def setzeStandard(self):
        """
        Der Stift wird zurckgesetzt. Der Stift befindet sich angehoben auf Position (0,0).
        """

        self.NORMALMODUS = 0
        self.RADIERMODUS=1
        self.WECHSELMODUS=2
        # Stiftposition
        self.zstiftx=0
        self.zstifty=0
        # Stiftzustand
        self.zhoch=1         # true
        # Stiftrichtung
        self.zwinkel=0
        # Normalmodus
        self.zschreibmodus = self.NORMALMODUS
        # self.normal() # Normalmodus 	
        self.farbe=weiss
        self.Linienbreite=1
        self.FuellMuster=0
        self.SchriftArt="" #"arial.ttf"
        self.SchriftStil=0
        self.SchriftGroesse=14

    def setzeFarbe(self,neue_farbe):
        """
        Setze Farbe auf den angebenen Farbwert.
        neue_farbe = (Konstanten: schwarz, weiss, rot, gruen, blau, gelb) oder ein
        Tupel der Form (r,g,b) mit Werten aus (0..255)
        """
        
        self.farbe=neue_farbe
        
    def setzeLinienbreite(self,breite):
        "Setzte die Linienbreite auf den Wert breite:int (1..15)"
        self.Linienbreite=breite

    def setzeFuellMuster(self,muster):
        "setze Fllung 0=aus, 1=an"
        self.FuellMuster=muster

    def setzeSchriftArt(self,art):
        "Lade Schriftart. art:string mit Werten: 'arial' oder 'cour'"
        self.SchriftArt=art

    def setzeSchriftStil(self,stil):
        "Setzte Schriftstil. stil:int mit 0=normal, 1=fett, 2=kursiv, 4=unterstrichen"
        self.SchriftStil=stil

    def setzeSchriftGroesse(self,groesse):
        "Setzte Schriftgroesse. groesse: int (10 oder jede andere positive Zahl)"
        self.SchriftGroesse=groesse

    def setzeModus(self,modus):
        pass
        
    def textBreite(self,text):
        "Liefert die breite einer Textausgabe"
        return len(text)*self.SchriftGroesse

    def zahlBreite(self,text):
        "Liefert die breite einer Textausgabe"
        return len(str(text))*self.SchriftGroesse

    def zeichneLinie(self, x1, y1, x2, y2):
        if self.zschreibmodus==self.NORMALMODUS:
      	  self.bildschirm.drawline((x1, y1), (x2, y2),self.farbe,self.Linienbreite)
        else:
      	  self.bildschirm.drawline((x1, y1), (x2, y2),(0,0,0),self.Linienbreite)
		
    def schreibe(self, ps):
        ps=str(ps)
        
        if self.zschreibmodus==self.NORMALMODUS: farbe=self.farbe
        else: farbe=schwarz
        
      	self.bildschirm.fonttext(ps,(self.zstiftx,self.zstifty),farbe,self.SchriftArt,self.SchriftGroesse,self.SchriftStil)
      	self.zstiftx=self.zstiftx + self.FontMetrics*len(ps)     	

    def zeichneKreis (self, pradius):
        """
        Zeichnet an der aktuellen Position einen Kreis mit dem Radius pradius.
        """      
        if self.zschreibmodus==self.NORMALMODUS:
          self.bildschirm.kreis((self.zstiftx,self.zstifty),pradius,self.farbe,self.FuellMuster)
        else:
          self.bildschirm.kreis((self.zstiftx,self.zstifty),pradius,(0,0,0),self.FuellMuster)
        


#######################################################################################
# Programmstck, um die Klasse Stift zu testen
#######################################################################################

if __name__ == '__main__':
  from bildschirm import *

  bildschirm=Bildschirm()
  test = Buntstift() 
  test.setzeFarbe(rot)
  
  print "Winkel:  ", test.zwinkel, "\vjetzt wird gedreht, um 45': "
  test.dreheUm(45)
  print "Winkel: ", test.zwinkel
  print "Stift - ",
  print "\vPosition: horizontal ", test.zstiftx, " vertikal ", test.zstifty
  print "         Zustand: ", 
  if test.zschreibmodus== test.NORMALMODUS: 
  	print "NORMALMODUS"
  elif test.zschreibmodus == test.RADIERMODUS:
        print "RADIERMODUS"
  elif test.zschreibmodus == test.WECHSELMODUS:
        print "WECHSELMODUS"
  
  print "         Pen: ",	
  if test.zhoch: print "    oben" 
  else: print "    unten"	
		
  test.runter()
  
  print "         Pen: ",	
  if test.zhoch: print "    oben" 
  else: print "    unten"	
  
  test.bewegeBis(320, 240)
  test.zeichneRechteck(150,150)
  
  test.setzeLinienbreite(5)
  test.setzeFarbe(gelb)
  test.bewegeBis(20, 40)
  test.zeichneRechteck(170,400)
  
  test.bewegeBis(420, 140)
  
  test.zeichneKreis(10)
  test.bewegeUm(-100)
  
  test.setzeFarbe(gruen)
  test.setzeSchriftGroesse(30)
  test.setzeSchriftStil(2)
  test.schreibe("Das musste mal geschrieben werden")

  test.zeichneKreis(80)
  
  test.radiere()
  test.wechsle()
 
  test.gibFrei()
  bildschirm.gibFrei()
  print "Ende!"
  