# coding: latin-1
"""
Diese Datei ist Teil der Stifte-und-Muse-Portierung fr Python,
entstanden im Rahmen der Diplomarbeit von Ingo Linkweiler,
verfentlicht unter der Creative Commons Lizenz by-nc-sa,
naehere Informationen dazu unter
http://creativecommons.org/licenses/by-nc-sa/3.0/de/.

 Erste Version: 14.01.2002 - 21.03.2003
 Autor: i.linkweiler@gmx.de
"""

from bildschirm import *
from stift import *
from maus import *
from tastatur import *

class EreignisAnwendung:
  """
  Eine Ereignis-Anwendung ist eine abstrakte Klasse als Basis eine Anwendung, die auf Ereignisse
  der Tastatur und Maus reagiert. Anfallende Ereignisse werden einer abstrakten Methode bergeben.
  Bei der Realisierung von Anwendungen mssen insbes. die Methoden 'bearbeiteTaste',
  'bearbeiteMausDruck' berschrieben werden.
  Die Methoden '__init__' und 'gibFrei' mssen aufgerufen werden.
  '__init__' sollte nicht berschrieben werden. Wird eine eigene init-Methode gebraucht,
  sollte ein anderer Name wie z.B. 'start' verwendet und diese explizit aufgerufen werden.
  """
  
  def __init__(self):
    """Erzeugen und Initialisieren der verwendeten Objekte"""
    self.bildschirm=Bildschirm()
    self.stift=Stift(self.bildschirm)
    self.maus=Maus(self.bildschirm)
    self.tastatur=Tastatur(self.bildschirm)

    self.beendet=0
    self.letzter_mausstatus=0
    self.letzte_mauspos=(0,0)

    
  def gibFrei(self):
    "Freigabe der verwendeten Objekte"
    self.tastatur.gibFrei()
    self.stift.gibFrei()
    self.maus.gibFrei()
    self.bildschirm.gibFrei()

  def hatBildschirm(self):
    "liefert den Bildschirm der Anwendung"
    return self.bildschirm

  def hatMaus(self):
    "liefert die Maus der Anwendung"
    return self.maus

  def hatTastatur(self):
    "liefert die Tastatur der Anwendung"
    return self.tastatur
    
  def fuehreAus(self):
    """
    Es wird auf Ereignisse gewartet und ggf. die Methoden [bearbeiteTaste, bearbeiteMausDruck,
    bearbeiteSpezialKlick, bearbeiteMausLos, bearbeiteMausBewegt] aufgerufen.
    """
    
    while not self.beendet:
      if self.tastatur.wurdeGedrueckt():      
        self.bearbeiteTaste(self.tastatur.zeichen())
      elif self.maus.istGedrueckt ()==1 and self.letzter_mausstatus==0:
        self.bearbeiteMausDruck(self.maus.hPosition(),self.maus.vPosition())
        self.letzter_mausstatus=1
      elif self.maus.istGedrueckt ()>1 and self.letzter_mausstatus==0:
        self.bearbeiteSpezialKlick(self.maus.hPosition(),self.maus.vPosition())
        self.letzter_mausstatus=1
      elif self.maus.istGedrueckt ()==0 and self.letzter_mausstatus==1:
        self.bearbeiteMausLos(self.maus.hPosition(),self.maus.vPosition())
        self.letzter_mausstatus=0
      elif self.letzte_mauspos<>self.maus.position():
        self.letzte_mauspos=self.maus.position()
        self.bearbeiteMausBewegt(self.maus.hPosition(),self.maus.vPosition())
      else:
        self.bearbeiteLeerlauf()

  def Beenden(self):
    "Die Anwendung wird beendet."
    self.beendet=1
      
  def bearbeiteTaste(self,zeichen):
    """Ein Tastatur-Ereignis ist eingetreten.Diese Methode kann neu definiert werden."""
    print "Ereignis Zeichen",zeichen
    if zeichen==chr(27): self.Beenden()

  def bearbeiteMausDruck(self,x,y):
    """Ein Mausdruck-Ereignis ist eingetreten.Diese Methode kann neu definiert werden."""
    print "Mausklick",x,y

  def bearbeiteSpezialKlick(self,x,y):
    """Ein Mausdruck-Ereignis ist eingetreten.Diese Methode kann neu definiert werden."""
    print "Spezialklick",x,y

  def bearbeiteMausBewegt(self,x,y):
    """Ein Maus-Bewegt-Ereignis ist eingetreten.Diese Methode kann neu definiert werden."""
    print "Maus bewegt",x,y

  def bearbeiteMausLos(self,x,y):
    """Ein Mausdruck-Ereignis ist eingetreten.Diese Methode kann neu definiert werden."""
    print "Mauslos",x,y

  def bearbeiteLeerlauf(self):
    """Kein Ereignis ist eingetreten, die Anwendung ist im Leerlauf. Diese Methode kann neu definiert werden."""
    pass


######################################################
# Testklasse
######################################################

class Test(EreignisAnwendung):
  "Test"
  pass

def main():
  "Anwendung erzeugen, starten und beenden."
  a=Test()
  a.fuehreAus()
  a.gibFrei()
  
    
if __name__ == '__main__': main()
