# coding: latin-1
"""
Diese Datei ist Teil der Stifte-und-Muse-Portierung fr Python,
entstanden im Rahmen der Diplomarbeit von Ingo Linkweiler,
verfentlicht unter der Creative Commons Lizenz by-nc-sa,
naehere Informationen dazu unter
http://creativecommons.org/licenses/by-nc-sa/3.0/de/.

 Erste Version: 14.01.2002 - 11.09.2002
 Autor: i.linkweiler@gmx.de
   
 Letzte nderung:
   16.01.2009 von Matthias Heming (matthias@familie-heming.de)
     Einfgen der Lizenzhinweise, Verschieben des Eingangskommentar
     in Server- und Client-Klasse
     
 TODO: Implementierung von Fllmustern
"""

import math
import sumwrapper

# Konstanten
weiss=(255,255,255)
schwarz=(0,0,0)

class Stift:
    """
    Der Stift ist ein Zeichenwerkzeug, dass sich auf dem Bildschirm bewegen kann.
    Er hat eine Position (Ursprung ist links oben) und eine Richtung (0'=rechts, Drehsinn math. positiv.)
    Der Stift kennt zwei Zustnde: gesenkt (dann wird geschrieben) und gehoben (dann wird
    der Stift bewegt, ohne zu schreiben).
    Zum Zeichnen kennt er zwei Modi: 'normal' zum Schreiben, und 'radieren' zum Loeschen bzw. Ausradieren.
    """

    # der folgende Wert ist empirisch ermittelt
    FontMetrics = 9
    NORMALMODUS = 0
    RADIERMODUS=1
    WECHSELMODUS=2
    zstiftx=0
    zstifty=0
    zhoch=1         # true
    zwinkel=0
    zschreibmodus=NORMALMODUS

    def __init__(self, kennt_bildschirm=None):
        """
        Der Stift wird initialisiert. Der Stift befindet sich angehoben auf Position (0,0).
        Als Parameter muss der zugehrende Bildschirm angegeben werden.
        """

        if kennt_bildschirm != None:
          self.bildschirm=kennt_bildschirm.bild
        else:
          self.bildschirm=sumwrapper.SCREEN

        if self.bildschirm==None:
          raise "Es ist kein Bildschirm verfuegbar."
        
        self.setzeStandard()
        
    def gibFrei(self):
        "Der Stift wird freigegeben und steht nicht mehr zur Verfgung."
      	pass

    def setzeStandard(self):
        """
        Der Stift wird zurckgesetzt. Der Stift befindet sich angehoben auf Position (0,0).
        """

        self.NORMALMODUS = 0
        self.RADIERMODUS=1
        self.WECHSELMODUS=2
        # Stiftposition
        self.zstiftx=0
        self.zstifty=0
        # Stiftzustand
        self.zhoch=1         # true
        # Stiftrichtung
        self.zwinkel=0
        # Normalmodus
        self.zschreibmodus = self.NORMALMODUS
	
    		
    def zeichneLinie(self, x1, y1, x2, y2):
        "Zeichne eine Linie von x1,y1 nach x2,y2"
        if self.zschreibmodus==self.NORMALMODUS:
      	  self.bildschirm.drawline((x1, y1), (x2, y2),weiss)
        else:
      	  self.bildschirm.drawline((x1, y1), (x2, y2),(0,0,0))

		
    def bewegeBis(self, px, py):    
        "Bewege den Stift nach px,py"

        if not self.zhoch:
           self.zeichneLinie(px, py, self.zstiftx, self.zstifty)
        self.zstiftx = px
        self.zstifty = py
    
    def bewegeUm(self, pl):    	
        "Bewege den Stift pl Einheiten in der aktuellen Richtung"

        w = self.zwinkel * math.pi/180
        x = self.zstiftx + pl * math.cos(w)
        y = self.zstifty - pl * math.sin(w)
        if not self.zhoch:  
           self.zeichneLinie(self.zstiftx, self.zstifty, x, y)
        self.zstiftx = x
        self.zstifty = y

    def dreheBis(self, pw):
        "Setzte den Drehwinkel auf pw:int (0..359)"
        self.zwinkel = pw
      	while (self.zwinkel < 0):
    	    self.zwinkel = self.zwinkel + 360
      	while (self.zwinkel >= 360 ):
    	    self.zwinkel = self.zwinkel-360
	      
    def dreheUm(self, pw):
        "Erhhe den Drehwinkel um pw:int (0..359)"

      	self.zwinkel = self.zwinkel + pw
      	while (self.zwinkel <0):
      	    self.zwinkel = self.zwinkel + 360
      	while (self.zwinkel >= 360 ):
      	    self.zwinkel = self.zwinkel - 360	      
  	      
    def schreibe(self, ps):
        """
        Gib einen Text an der aktuellen Position aus.
        Der Stift steht anschliessend am Ende des Textes.
        """

        ps=str(ps)
        if self.zschreibmodus==self.NORMALMODUS: farbe=weiss
        else: farbe=schwarz
       
        self.bildschirm.text(ps,(self.zstiftx,self.zstifty),farbe)
        self.zstiftx=self.zstiftx + self.FontMetrics*len(ps)     	

    def schreibeText(self, pText):
        self.schreibe(pText)

    def schreibeZahl(self, pZahl):
        self.schreibe(pZahl)


    def hoch(self):
        "Hebe den Stift"
        self.zhoch=1 # true	
	
    def runter(self):	
        "senke den Stift"
        self.zhoch=0 # false	
     	      
    def normal(self):
        "aktiviere den normalen Schreibmodus"
        self.zschreibmodus=self.NORMALMODUS
	
    def radiere(self):
        "aktiviere den Radiermodus"
        #self.bildschirm.screenrefresh()
        self.zschreibmodus=self.RADIERMODUS
    
    def wechsle(self):
        "(nicht voll implementiert, aktiviert ebenfalls den Radiermodus)"
      	self.zschreibmodus=self.WECHSELMODUS

    def hPosition(self):
        "liefert die horizontale X-Position"
        return self.zstiftx
	
    def vPosition(self):
        "liefert die vertikake Y-Position"
        return self.zstifty
	
    def winkel(self):
        "liefert den aktuellen Winkel"
        return self.zwinkel
	
    def zeichneRechteck(self, pbreite, phoehe):
        """
        Zeichnet an der aktuellen Position ein Rechteck nach rechts unten.
        pbreite: int, phoehe: int
        """
        
        self.zeichneLinie(self.zstiftx,self.zstifty,self.zstiftx+pbreite,self.zstifty) #oben
        self.zeichneLinie(self.zstiftx,self.zstifty,self.zstiftx,self.zstifty+phoehe) #links
        self.zeichneLinie(self.zstiftx+pbreite,self.zstifty,self.zstiftx+pbreite,self.zstifty+phoehe) #rechts
        self.zeichneLinie(self.zstiftx+pbreite,self.zstifty+phoehe,self.zstiftx,self.zstifty+phoehe)
	
    def zeichneKreis (self, pradius):
        """
        Zeichnet an der aktuellen Position einen Kreis mit dem Radius pradius.
        """ 
        if self.zschreibmodus==self.NORMALMODUS:
          self.bildschirm.kreis((self.zstiftx,self.zstifty),pradius,weiss)
        else:
          self.bildschirm.kreis((self.zstiftx,self.zstifty),pradius,(0,0,0))


#######################################################################################
# Programmstck, um die Klasse Stift zu testen
#######################################################################################

if __name__ == '__main__':
  from bildschirm import *
  bildschirm=Bildschirm()
  test = Stift()
  
  print "Winkel:  ", test.zwinkel, "\vjetzt wird gedreht, um 45': "
  test.dreheUm(45)
  print "Winkel: ", test.zwinkel
  print "Stift - ",
  print "\vPosition: horizontal ", test.zstiftx, " vertikal ", test.zstifty
  print "         Zustand: ", 
  if test.zschreibmodus== test.NORMALMODUS: 
  	print "NORMALMODUS"
  elif test.zschreibmodus == test.RADIERMODUS:
        print "RADIERMODUS"
  elif test.zschreibmodus == test.WECHSELMODUS:
        print "WECHSELMODUS"
  
  print "         Pen: ",	
  if test.zhoch: print "    oben" 
  else: print "    unten"	
		
  test.runter()
  
  print "         Pen: ",	
  if test.zhoch: print "    oben" 
  else: print "    unten"	
  
  test.bewegeBis(320, 240)
  test.zeichneRechteck(150,150)
  test.bewegeBis(20, 40)
  test.zeichneRechteck(170,400)
  
  test.bewegeBis(420, 140)
  
  test.zeichneKreis(10)
  test.bewegeUm(-40)  
  
  for i in range(100):
    test.bewegeUm(160)
    test.dreheUm(145)
  
  test.bewegeBis(40,40)
  test.schreibe("Das musste mal geschrieben werden")

  test.zeichneKreis(80)
  
  test.radiere()
  test.wechsle()
 
  test.gibFrei()
  bildschirm.gibFrei()
  print "Ende!"
