# coding: latin-1
"""
Diese Datei ist Teil der Stifte-und-Muse-Portierung fr Python,
entstanden im Rahmen der Diplomarbeit von Ingo Linkweiler,
verfentlicht unter der Creative Commons Lizenz by-nc-sa,
naehere Informationen dazu unter
http://creativecommons.org/licenses/by-nc-sa/3.0/de/.

 Erste Version: 21.01.2002 - 09.03.2003
 Autor: i.linkweiler@gmx.de
 
 Modifiziert von Matthias Heming
 matthias@familie-heming.de
 Version vom 24.09.2008
 
 Es wurden folgende Funktionen in der Klasse "Bildserver" hinzugefuegt:
  def drawpolyline(self,punkteliste,farbe,linienstaerke)
  def drawpolygon(self,punkteliste,farbe,linienstaerke)
  def drawellipse(self,pos,radiusx,radiusy,farbe,linienstaerke=1,drehwinkel=0)
  def drawkreis(self,pos,radius,farbe,linienstaerke=1)
  def drawtext(self,text,pos,farbe,font,groesse,drehwinkel=0,fett=False,kursiv=False,unterstrichen=False)
"""

import time
# Importiere benutze Module
import random, os.path, sys
import pygame, pygame.image, pygame.transform, pygame.sprite, pygame.draw, pygame.time
from pygame.locals import *
from math import *

from threading import *

# Versuche, pygame optional modules zu importieren
try:
    import pygame.font as font
except ImportError:
    print 'Warning, no fonts'
    font = None
try:
    import pygame.mixer
except ImportError:
    print 'Warning, no sound'
    pygame.mixer = None

try:
  _PATH=os.path.dirname(__file__)
except:
  _PATH=""

global SCREEN
SCREEN=None

# Konstanten
SCREENRECT = Rect(0, 0, 640, 480)

weiss=(255,255,255)
schwarz=(0,0,0)
rot=(255,0,0)
gruen=(0,255,0)
blau=(0,0,255)
gelb=(255,255,0)
grau=(100,100,100)


def load_image_key(name, colorkey=None):
    "intern: Lade ein Bild, colorkey ist transparente Farbe"
    fullname = os.path.join(_PATH,'data', name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error, message:
        print 'Cannot load image:', name
        #raise SystemExit, message
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
        
    return image, image.get_rect()

def load_image(file):
    "intern: Lade ein Bild"

    file = os.path.join(_PATH,'data', file)
    try:
        surface = pygame.image.load(file)
    except pygame.error:
        print 'Could not load image "%s" %s'%(file, pygame.get_error())
        surface = pygame.Surface(SCREENRECT.size)
    return surface.convert()

def load_images(*files):
    "intern: Lade Bildliste"
    return [load_image(file) for file in files]

################################################
# Bildserver
################################################

class Bildserver:
  """
   derBild Server ist die Zentrale interne Komponente zur Darstellung der Grafiken
   und Kommunikation mit dem Betriebssystem.
   Dies geschieht durch Kommunikation mit der Bibliothek PyGame.
  """

  def screenrefresh(self):
      "intern: Bildschirm aktualisieren, damit auch im Interpreter der Redraw funktioniert."
      self.changed=0
      self.allsprites.update()
      self.screen.blit(self.background, (0,0))
      self.allsprites.draw(self.screen)     
      pygame.display.flip()

  def eventloop(self):
    "intern: Abfrage der Systemereignisse Tastatur, Maus...."
    ev=pygame.event.poll()
    # Events bearbeiten
    while ev.type<>NOEVENT:
      if ev.type==KEYDOWN:
        c=unicode(ev.unicode).encode("cp437", "replace")
        self.texteingaben=self.texteingaben+c
        # print c,
      elif ev.type==MOUSEMOTION:
        self.mausposxy=ev.pos
        #print self.mausposxy
      elif ev.type==MOUSEBUTTONDOWN:
        self.mausbutton=ev.button
        #print self.mausbutton
      elif ev.type==MOUSEBUTTONUP:
        self.mausbutton=0
        #print self.mausbutton
      elif str(ev.type)=="12":
        print "sys-exit"
        if self.server.srvactive==1:
          self.server.kill()
          self.beenden=1
        else:
          pygame.display.iconify() 
      ev=pygame.event.poll()

    # auf 1/100sec synchronisieren
    while pygame.time.get_ticks()<self.ticks:
      time.sleep(0.01)
    self.ticks=pygame.time.get_ticks()+10 #+10ms

    # Wenn Bild gendert, aktualisieren
    if self.changed: self.screenrefresh()
    self.sleepcounter=0

  def subthread(self):
    """
    intern: Thread, aktualisiert 1*/sec das Ausgabefenster, damit auch im Interpreter
    der Redraw funktioniert.
    """

    while 1:
        try:
          self.changed=1
          self.eventloop()
        except:
          #print "Fenster wurde geschlossen."
          raise SystemExit
          #print "weg ist er..."

        time.sleep(1)

  def thread(self):
      "intern: starte den Redraw-Thread"
      self.st = Thread(target = self.subthread)   # dies...
      self.st.start()                        # ...ist schon alles!

  def reset(self):
      self.changed=0
      self.raus=0
      self.beenden=0
      self.aktive_sprites=[]
      self.letzte_pos=(0,0)
      self.texteingaben=""
      self.mausposxy=(0,0)
      self.mausbutton=0
      self.active=0 
      
      self.sleepcounter=0   

  def __init__(self,hintergrundbild=""):
      """
      nachher: Der Bildschirm ist initialisiert.
      'hintergrundbild' ist ein optionaler Dateiname einer Grafikdatei als Bildschirmhintergrund.    
      """

      global SCREEN
      if SCREEN != None: raise "Es darf nur ein Bildschirm verwendet werden."

      self.reset()
      
      # Initialisiere pygame und ffne ein Fenster.
      pygame.init()
      self.ticks=pygame.time.get_ticks()+20

      # display mode setzen
      winstyle = 0 # |FULLSCREEN
      bestdepth = pygame.display.mode_ok(SCREENRECT.size, winstyle, 16)
      # Hintergrundbild laden
      self.screen = pygame.display.set_mode(SCREENRECT.size, winstyle, bestdepth)
   
      pygame.display.set_caption("Stifte und Maeuse")
      self.setzeHintergrund(hintergrundbild)

      self.aktive_sprites=[]
      self.allsprites=pygame.sprite.RenderPlain(self.aktive_sprites)
       
      self.eventloop()
      self.setzeHintergrund("sum.jpg")     
      try:
        self.snd=pygame.mixer.Sound("./data/sum.wav")
        self.snd.play()
      except:
        print "Kein Sound gefunden."

      self.loescheAlles()
      if hintergrundbild != "": self.setzeHintergrund(hintergrundbild)

      SCREEN=self
      
      self.eventloop()     
      self.thread()
      self.changed=1

  def __del__(self):
      "intern: Destruktor, nur zur Sicherheit"
      print "del"
      self.ende()
      
  def breite(self):
      "liefert die breite der Zeichenebene"
      return 639

  def hoehe(self):
      "liefert die hoehe der Zeichenebene"
      return 479

  def loescheAlles(self):
      "loescht die Zeichenebene"
      self.background = pygame.Surface(SCREENRECT.size)
      self.screen.blit(self.background, (0,0))
      self.changed=1

  def setzeHintergrund(self,dateiname):
      "Lade einen Hintergrund und zeige diesen im Fenster."
      self.background = pygame.Surface(SCREENRECT.size)
      if dateiname<>"":
        bgdtile = load_image(dateiname)
        self.background = pygame.Surface(SCREENRECT.size)
        for x in range(0, SCREENRECT.width, bgdtile.get_width()):
            self.background.blit(bgdtile, (x, 0))

      self.screen.blit(self.background, (0,0))
      self.changed=1
      
  def gibFrei(self):
      "Das Ausgabefenster wird abgebaut und auf einen Tastendruck zum Programmende gewartet."
      print "*** Sum wird beendet ***"
      self.screenrefresh()
      time.sleep(1)
      self.ende()

  def mausstatus(self):
      "true, wenn eine Maustaste gedrckt."
      if self.changed or self.sleepcounter>100: self.eventloop()
      self.sleepcounter=self.sleepcounter+1
      return self.mausbutton

  def mauspos(self):
      "liefert Mausposition als tupel."
      if self.changed or self.sleepcounter>100: self.eventloop()
      self.sleepcounter=self.sleepcounter+1
      return self.mausposxy

  def eingaben(self):
      "Liefert die letzten Tastatureingaben"
      if self.changed or self.sleepcounter>100: self.eventloop()
      self.sleepcounter=self.sleepcounter+1
      r=self.texteingaben
      self.texteingaben=""
      return r

  def ende(self):
      "Ausgabefenster schlieen."
      self.st = 0
      pygame.quit()
      
      global SCREEN
      SCREEN=None

  def setpixel(self,pos,farbe):
      "Setze ein Pixel an Position pos:tupel in der Farbe farbe:3xtupel."
      if (pos[0]>=0) and (pos[1]>=0) and (pos[0]<640) and (pos[1]<480):
        self.screen.set_at(pos,farbe)
        self.background.set_at(pos,farbe)
        self.changed=1
      self.letzte_pos=pos

  def drawline(self,pos1,pos2,farbe,breite=1):
      "Zeichnet eine Linie von pos1:tupel nach pos2:tupel in farbe:tupel und breite:int"
      pygame.draw.line(self.background, farbe, pos1, pos2, breite)
      pygame.draw.line(self.screen, farbe, pos1, pos2, breite)
      self.changed=1
      
  ### Hinzugefuegt von Matthias Heming
  def drawpolyline(self,punkteliste,farbe,linienstaerke):
      """
      Zeichnet einen Polygonzug in farbe:tupel durch die
      in punkteliste angegebenen Punkte.
      """
      linienstaerke = int(linienstaerke)
      pygame.draw.lines(self.background,farbe,False,punkteliste,linienstaerke)
      pygame.draw.lines(self.screen,farbe,False,punkteliste,linienstaerke)
      self.changed=1
  
  def drawpolygon(self,punkteliste,farbe,linienstaerke):
      """
      Zeichnet ein geschlossenes farbe:tupel Polygon durch die
      in punkteliste angegebenen Punkte. Fuer linienstaerke=0
      ist das Polygon gefuellt.
      """
      linienstaerke = int(linienstaerke)
      pygame.draw.polygon(self.background,farbe,punkteliste,linienstaerke)
      pygame.draw.polygon(self.screen,farbe,punkteliste,linienstaerke)
      self.changed=1
      
  def drawellipse(self,pos,radiusx,radiusy,farbe,linienstaerke=1,drehwinkel=0):
      """
      Zeichnet eine Ellipse mit Mittelpunkt pos:tupel und vorgegebenen
      Radien radiusx:int radiusy:int. Fuer linienstaerke=0 ist die Ellipse
      gefuellt.
      Die Ellipse wird um drehwinkel:int Grad gedreht gezeichnet. Wo negative
      Zahlen eine Drehrichtung im Uhrzeigersinn bedeuten.
      """
      linienstaerke = int(linienstaerke)
      # Temporaeres Surface-Obejekt mir per-pixel alpha erstellen
      nsurf = pygame.Surface((radiusx*2,radiusy*2),pygame.SRCALPHA)
      # Ellipse zeichnen
      pygame.draw.ellipse(nsurf,farbe,(0,0,2*radiusx,2*radiusy),linienstaerke)
      # Falls gewuenscht, Surface drehen
      if drehwinkel!=0: nsurf = pygame.transform.rotate(nsurf,drehwinkel)
      # Rechteck fuer Kopie auf Ziel-Surface erstellen
      targetrect = nsurf.get_rect(center=pos)
      # Kopie und damit eigentliche Zeichnung auf self.background und self.screen
      self.background.blit(nsurf,targetrect)
      self.screen.blit(nsurf,targetrect)
      self.changed=1
      
  def drawkreis(self,pos,radius,farbe,linienstaerke=1):
      """
      Zeichnet einen Kreis mit Mittelpunkt pos:tupel und radius:int mit
      linienstaerke:int Dicke. Ist linienstaerke=0, so ist der Kreis gefuellt.
      """
      linienstaerke = int(linienstaerke)
      self.drawellipse(pos,radius,radius,farbe,linienstaerke)
      
  def drawtext(self,text,pos,farbe,font,groesse,drehwinkel=0,fett=False,kursiv=False,unterstrichen=False):
      """
      Zeichnet Text in angegebenem Schriftstil. Dabei bezeichnet pos:tupel, den Mittelpunkt
      der BoundingBox des gerenderten Textes. Dabei wird der Text um drehwinkel:int Grad
      gedreht.
      """
      groesse = int(groesse)
      if font=="": font="default.ttf"
      font=pygame.font.Font(os.path.join(_PATH,font),groesse)
      if fett: font.set_bold(true)
      if kursiv: font.set_italic(1)
      if unterstrichen: font.set_underline(1)
      text=font.render(text,0,farbe)
      if drehwinkel != 0: text = pygame.transform.rotate(text,drehwinkel)
      targetrect = text.get_rect(center=pos)
      self.background.blit(text,targetrect)
      self.screen.blit(text,targetrect)
      self.changed=1
  ### Ende der hinzugefgten Methoden
    
  def kreis(self,pos,radius,farbe,fuellung=0):
      " Zeichnet einen Kreis."
      if fuellung>0: fm=0
      else: fm=1
      pygame.draw.circle(self.background, farbe, pos, radius, fm)
      pygame.draw.circle(self.screen, farbe, pos, radius, fm)     
      self.changed=1

  def text(self,text,pos,farbe):
      "Gibt Text aus."
      font=pygame.font.Font(os.path.join(_PATH,"default.ttf"),14)    
      text=font.render(text,0,farbe)
      self.background.blit(text,pos)
      self.screen.blit(text,pos)  
      self.changed=1

  def fonttext(self,text,pos,farbe,font,groesse,flag=0):
      "Gibt Text mit font aus."
      if font=="": font="default.ttf"
      font=pygame.font.Font(os.path.join(_PATH,font),groesse)
      if flag==1:
        font.set_bold(1)
      if flag==2:
        font.set_italic(1)
      if flag==4:
        font.set_underline(1)
      text=font.render(text,0,farbe)
      self.background.blit(text,pos)
      self.screen.blit(text,pos)
      self.changed=1

  def collision(self,sprite1id,sprite2id):
      "Teste Kollision zwischen spriteid1 und 2."
      for such in self.aktive_sprites:
        if such.id==sprite1id: sprite1=such
        if such.id==sprite2id: sprite2=such

      r=0
      hitbox = sprite1.rect.inflate(-5, -5)
      r=r+hitbox.colliderect(sprite2.rect)
      return r
     
  def loesche_sprite(self,loeschspriteid):
      "loesche das angegebene Sprite"
      pass

  def loesche_sprite(self,loeschsprite):
      "intern: loesche das angegebene Sprite"
      self.aktive_sprites.remove(loeschsprite)
      self.allsprites=pygame.sprite.RenderPlain(self.aktive_sprites)
      self.changed=1
      
  def setpos(self,id,pos):
      "Setzte das Sprite id an die Position pos"
      for sp in self.aktive_sprites:
        if sp.id == id:
          sp.setpos(pos)
          break
      self.changed=1
     
     
  def add_sprite(self,dateiname):
      "Erzeuge ein neues Sprite aus Bilddatei und fge es dem Bildschirm zu"
      sp=Sprite(dateiname,self)
      self.make_sprite(sp)
      self.changed=1
      return sp.id

  def dreheBis(self,id,winkel):
      "Setzte das Sprite id an die Position pos"
      for sp in self.aktive_sprites:
        if sp.id == id:
          sp.dreheBis(winkel)
          break
      self.changed=1
 
  def make_sprite(self,neues_sprite): 
      "intern: Fge ein Sprite in die Spriteliste ein."
      self.aktive_sprites.append(neues_sprite)
      self.allsprites=pygame.sprite.RenderPlain(self.aktive_sprites)
      self.changed=1



################################################
# intern: Klasse Sprite
################################################

class Sprite(pygame.sprite.Sprite):
    """
    Diese neue Klasse Sprite stellt eine auf der Ausgabeflche frei bewegbare Figur dar.
    """
  
    def __init__(self,bild,fenster):
        "identisch zu laden"
        self.laden(bild,fenster)
    
    def laden(self,bilddatei,fenster):
        """
        Vorbereiten des Sprites: Das Bild (jpg/bmp) wird geladen und das Sprite ist vorbereitet.
        bilddatei:Dateiname als string; fenster: zugehriger Bildschirm
        """

        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
        self.image, self.rect = load_image_key(bilddatei,-1)
        self.oldimage = self.image
        self.rect = self.image.get_rect()
        self.pos=(random.randrange(5,500),40)
        self.move=(random.choice((1,5)),1)
        self.winkel=0
        self.fenster=fenster
        fenster.make_sprite(self)
        self.id=bilddatei+str(random.randint(1000,9999))

    def update(self):
        pass

    def bewegeBis(self,x,y):
        "Bewege das Sprite an die Position x,y: int"
        self.setpos((x,y))
        
    def hPosition(self):
        "liefert die aktuelle x-Position"
        return self.pos[0]
    def vPosition(self):
        "liefert die aktuelle y-Position"
        return self.pos[1]

    def setpos(self,pos):
        "intern: Setzte Sprite an Position pos: tupel"
        self.pos=pos
        # self.rect.midtop = self.pos    
        self.image=self.oldimage
        if self.winkel>0:
          winkel = self.winkel
          #PyGame-Bug workaround: 90er Winkel crashen!
          if winkel==90 or winkel==180 or winkel==270: winkel=winkel+1
          self.image=pygame.transform.rotate(self.image, winkel)
        if self.move[0]<0:
          self.image=pygame.transform.flip(self.image, 1, 0)
 
        self.rect = self.image.get_rect()       
        self.rect.center = self.pos  

    def dreheBis(self,neuwinkel):
        "Drehe das Spritebild auf den angegebenen Drehwinkel. neuwinkel:int"
        self.winkel=neuwinkel % 360
  
    def loeschen(self):
        "Entferne das Sprite vom Bildschirm."
        self.fenster.loesche_sprite(self)
        self.kill()
      
      
def main():
    "Testprogramm."
    bild=Bildserver()
    for i in range(20):
      farbe=(i*10,250-i*10,200)
      bild.text("Hallo, Bildschirmtest!",(10+i,10+i*16),farbe)      
      bild.drawline((i*30,10),(600-i*30,400),farbe,5)
    bild.kreis((320,200),100,gelb)
    #bild.
    bild.gibFrei()
        
# call the "main" function if running this script
if __name__ == '__main__': main()
